@@ -1,9 +1,9 @@
-/* A Bison parser, made by GNU Bison 2.4.2.  */
+/* A Bison parser, made by GNU Bison 2.4.3.  */
 
 /* Skeleton interface for Bison's Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989-1990, 2000-2006, 2009-2010 Free Software
-   Foundation, Inc.
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+   2009, 2010 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -112,120 +112,123 @@
      VOID_TOK = 329,
      WHILE = 330,
      IDENTIFIER = 331,
-     FLOATCONSTANT = 332,
-     INTCONSTANT = 333,
-     UINTCONSTANT = 334,
-     BOOLCONSTANT = 335,
-     FIELD_SELECTION = 336,
-     LEFT_OP = 337,
-     RIGHT_OP = 338,
-     INC_OP = 339,
-     DEC_OP = 340,
-     LE_OP = 341,
-     GE_OP = 342,
-     EQ_OP = 343,
-     NE_OP = 344,
-     AND_OP = 345,
-     OR_OP = 346,
-     XOR_OP = 347,
-     MUL_ASSIGN = 348,
-     DIV_ASSIGN = 349,
-     ADD_ASSIGN = 350,
-     MOD_ASSIGN = 351,
-     LEFT_ASSIGN = 352,
-     RIGHT_ASSIGN = 353,
-     AND_ASSIGN = 354,
-     XOR_ASSIGN = 355,
-     OR_ASSIGN = 356,
-     SUB_ASSIGN = 357,
-     INVARIANT = 358,
-     LOWP = 359,
-     MEDIUMP = 360,
-     HIGHP = 361,
-     SUPERP = 362,
-     PRECISION = 363,
-     VERSION = 364,
-     EXTENSION = 365,
-     LINE = 366,
-     COLON = 367,
-     EOL = 368,
-     INTERFACE = 369,
-     OUTPUT = 370,
-     PRAGMA_DEBUG_ON = 371,
-     PRAGMA_DEBUG_OFF = 372,
-     PRAGMA_OPTIMIZE_ON = 373,
-     PRAGMA_OPTIMIZE_OFF = 374,
-     LAYOUT_TOK = 375,
-     ASM = 376,
-     CLASS = 377,
-     UNION = 378,
-     ENUM = 379,
-     TYPEDEF = 380,
-     TEMPLATE = 381,
-     THIS = 382,
-     PACKED_TOK = 383,
-     GOTO = 384,
-     INLINE_TOK = 385,
-     NOINLINE = 386,
-     VOLATILE = 387,
-     PUBLIC_TOK = 388,
-     STATIC = 389,
-     EXTERN = 390,
-     EXTERNAL = 391,
-     LONG_TOK = 392,
-     SHORT_TOK = 393,
-     DOUBLE_TOK = 394,
-     HALF = 395,
-     FIXED_TOK = 396,
-     UNSIGNED = 397,
-     INPUT_TOK = 398,
-     OUPTUT = 399,
-     HVEC2 = 400,
-     HVEC3 = 401,
-     HVEC4 = 402,
-     DVEC2 = 403,
-     DVEC3 = 404,
-     DVEC4 = 405,
-     FVEC2 = 406,
-     FVEC3 = 407,
-     FVEC4 = 408,
-     SAMPLER2DRECT = 409,
-     SAMPLER3DRECT = 410,
-     SAMPLER2DRECTSHADOW = 411,
-     SIZEOF = 412,
-     CAST = 413,
-     NAMESPACE = 414,
-     USING = 415,
-     ERROR_TOK = 416,
-     COMMON = 417,
-     PARTITION = 418,
-     ACTIVE = 419,
-     SAMPLERBUFFER = 420,
-     FILTER = 421,
-     IMAGE1D = 422,
-     IMAGE2D = 423,
-     IMAGE3D = 424,
-     IMAGECUBE = 425,
-     IMAGE1DARRAY = 426,
-     IMAGE2DARRAY = 427,
-     IIMAGE1D = 428,
-     IIMAGE2D = 429,
-     IIMAGE3D = 430,
-     IIMAGECUBE = 431,
-     IIMAGE1DARRAY = 432,
-     IIMAGE2DARRAY = 433,
-     UIMAGE1D = 434,
-     UIMAGE2D = 435,
-     UIMAGE3D = 436,
-     UIMAGECUBE = 437,
-     UIMAGE1DARRAY = 438,
-     UIMAGE2DARRAY = 439,
-     IMAGE1DSHADOW = 440,
-     IMAGE2DSHADOW = 441,
-     IMAGEBUFFER = 442,
-     IIMAGEBUFFER = 443,
-     UIMAGEBUFFER = 444,
-     ROW_MAJOR = 445
+     TYPE_IDENTIFIER = 332,
+     NEW_IDENTIFIER = 333,
+     FLOATCONSTANT = 334,
+     INTCONSTANT = 335,
+     UINTCONSTANT = 336,
+     BOOLCONSTANT = 337,
+     FIELD_SELECTION = 338,
+     LEFT_OP = 339,
+     RIGHT_OP = 340,
+     INC_OP = 341,
+     DEC_OP = 342,
+     LE_OP = 343,
+     GE_OP = 344,
+     EQ_OP = 345,
+     NE_OP = 346,
+     AND_OP = 347,
+     OR_OP = 348,
+     XOR_OP = 349,
+     MUL_ASSIGN = 350,
+     DIV_ASSIGN = 351,
+     ADD_ASSIGN = 352,
+     MOD_ASSIGN = 353,
+     LEFT_ASSIGN = 354,
+     RIGHT_ASSIGN = 355,
+     AND_ASSIGN = 356,
+     XOR_ASSIGN = 357,
+     OR_ASSIGN = 358,
+     SUB_ASSIGN = 359,
+     INVARIANT = 360,
+     LOWP = 361,
+     MEDIUMP = 362,
+     HIGHP = 363,
+     SUPERP = 364,
+     PRECISION = 365,
+     VERSION = 366,
+     EXTENSION = 367,
+     LINE = 368,
+     COLON = 369,
+     EOL = 370,
+     INTERFACE = 371,
+     OUTPUT = 372,
+     PRAGMA_DEBUG_ON = 373,
+     PRAGMA_DEBUG_OFF = 374,
+     PRAGMA_OPTIMIZE_ON = 375,
+     PRAGMA_OPTIMIZE_OFF = 376,
+     PRAGMA_INVARIANT_ALL = 377,
+     LAYOUT_TOK = 378,
+     ASM = 379,
+     CLASS = 380,
+     UNION = 381,
+     ENUM = 382,
+     TYPEDEF = 383,
+     TEMPLATE = 384,
+     THIS = 385,
+     PACKED_TOK = 386,
+     GOTO = 387,
+     INLINE_TOK = 388,
+     NOINLINE = 389,
+     VOLATILE = 390,
+     PUBLIC_TOK = 391,
+     STATIC = 392,
+     EXTERN = 393,
+     EXTERNAL = 394,
+     LONG_TOK = 395,
+     SHORT_TOK = 396,
+     DOUBLE_TOK = 397,
+     HALF = 398,
+     FIXED_TOK = 399,
+     UNSIGNED = 400,
+     INPUT_TOK = 401,
+     OUPTUT = 402,
+     HVEC2 = 403,
+     HVEC3 = 404,
+     HVEC4 = 405,
+     DVEC2 = 406,
+     DVEC3 = 407,
+     DVEC4 = 408,
+     FVEC2 = 409,
+     FVEC3 = 410,
+     FVEC4 = 411,
+     SAMPLER2DRECT = 412,
+     SAMPLER3DRECT = 413,
+     SAMPLER2DRECTSHADOW = 414,
+     SIZEOF = 415,
+     CAST = 416,
+     NAMESPACE = 417,
+     USING = 418,
+     ERROR_TOK = 419,
+     COMMON = 420,
+     PARTITION = 421,
+     ACTIVE = 422,
+     SAMPLERBUFFER = 423,
+     FILTER = 424,
+     IMAGE1D = 425,
+     IMAGE2D = 426,
+     IMAGE3D = 427,
+     IMAGECUBE = 428,
+     IMAGE1DARRAY = 429,
+     IMAGE2DARRAY = 430,
+     IIMAGE1D = 431,
+     IIMAGE2D = 432,
+     IIMAGE3D = 433,
+     IIMAGECUBE = 434,
+     IIMAGE1DARRAY = 435,
+     IIMAGE2DARRAY = 436,
+     UIMAGE1D = 437,
+     UIMAGE2D = 438,
+     UIMAGE3D = 439,
+     UIMAGECUBE = 440,
+     UIMAGE1DARRAY = 441,
+     UIMAGE2DARRAY = 442,
+     IMAGE1DSHADOW = 443,
+     IMAGE2DSHADOW = 444,
+     IMAGEBUFFER = 445,
+     IIMAGEBUFFER = 446,
+     UIMAGEBUFFER = 447,
+     ROW_MAJOR = 448
    };
 #endif
 
@@ -272,7 +275,7 @@ typedef union YYSTYPE
 
 
 /* Line 1685 of yacc.c  */
-#line 276 "glsl_parser.h"
+#line 279 "glsl_parser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
