@@ -698,8 +698,8 @@ convertPalette(FxU32 data[256], const struct gl_color_table *table)
         return GR_TEXTABLE_PALETTE_6666_EXT;
     default:
         /* XXX fixme: how can this happen? */
-        _mesa_error(NULL, GL_INVALID_ENUM, "convertPalette: table->Format == %s",
-                                           _mesa_lookup_enum_by_nr(table->Format));
+        _mesa_error(NULL, GL_INVALID_ENUM, "convertPalette: table->_BaseFormat == %s",
+                                           _mesa_lookup_enum_by_nr(table->_BaseFormat));
         return GR_TEXTABLE_PALETTE;
     }
 }
