@@ -1,5 +1,4 @@
 /*
- *
  * Copyright (C) 2004  David Airlie   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,14 +19,12 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-#include "glheader.h"
-#include "colormac.h"
-#include "context.h"
-#include "atifragshader.h"
-#include "macros.h"
-#include "program.h"
-
-#include "s_atifragshader.h"
+#include "main/glheader.h"
+#include "main/colormac.h"
+#include "main/macros.h"
+#include "main/atifragshader.h"
+#include "swrast/s_atifragshader.h"
+#include "swrast/s_context.h"
 
 
 /**
@@ -49,17 +46,12 @@ static void
 fetch_texel(GLcontext * ctx, const GLfloat texcoord[4], GLfloat lambda,
 	    GLuint unit, GLfloat color[4])
 {
-   GLchan rgba[4];
    SWcontext *swrast = SWRAST_CONTEXT(ctx);
 
    /* XXX use a float-valued TextureSample routine here!!! */
    swrast->TextureSample[unit](ctx, ctx->Texture.Unit[unit]._Current,
                                1, (const GLfloat(*)[4]) texcoord,
-                               &lambda, &rgba);
-   color[0] = CHAN_TO_FLOAT(rgba[0]);
-   color[1] = CHAN_TO_FLOAT(rgba[1]);
-   color[2] = CHAN_TO_FLOAT(rgba[2]);
-   color[3] = CHAN_TO_FLOAT(rgba[3]);
+                               &lambda, (GLfloat (*)[4]) color);
 }
 
 static void
@@ -90,10 +82,11 @@ apply_swizzle(GLfloat values[4], GLuint swizzle)
       break;
    case GL_SWIZZLE_STQ_DQ_ATI:
 /* make sure q is not 0 to avoid problems later with infinite values (texture lookup)? */
-      if (q == 0.0F) q = 0.000000001;
+      if (q == 0.0F)
+         q = 0.000000001F;
       values[0] = s / q;
       values[1] = t / q;
-      values[2] = 1 / q;
+      values[2] = 1.0F / q;
       break;
    }
    values[3] = 0.0;
@@ -145,7 +138,7 @@ apply_src_mod(GLint optype, GLuint mod, GLfloat * val)
 	 val[i] = 1 - val[i];
 
       if (mod & GL_BIAS_BIT_ATI)
-	 val[i] = val[i] - 0.5;
+	 val[i] = val[i] - 0.5F;
 
       if (mod & GL_2X_BIT_ATI)
 	 val[i] = 2 * val[i];
@@ -179,27 +172,27 @@ apply_dst_mod(GLuint optype, GLuint mod, GLfloat * val)
 	 val[i] = 8 * val[i];
 	 break;
       case GL_HALF_BIT_ATI:
-	 val[i] = val[i] * 0.5;
+	 val[i] = val[i] * 0.5F;
 	 break;
       case GL_QUARTER_BIT_ATI:
-	 val[i] = val[i] * 0.25;
+	 val[i] = val[i] * 0.25F;
 	 break;
       case GL_EIGHTH_BIT_ATI:
-	 val[i] = val[i] * 0.125;
+	 val[i] = val[i] * 0.125F;
 	 break;
       }
 
       if (has_sat) {
-	 if (val[i] < 0.0)
-	    val[i] = 0;
-	 else if (val[i] > 1.0)
-	    val[i] = 1.0;
+	 if (val[i] < 0.0F)
+	    val[i] = 0.0F;
+	 else if (val[i] > 1.0F)
+	    val[i] = 1.0F;
       }
       else {
-	 if (val[i] < -8.0)
-	    val[i] = -8.0;
-	 else if (val[i] > 8.0)
-	    val[i] = 8.0;
+	 if (val[i] < -8.0F)
+	    val[i] = -8.0F;
+	 else if (val[i] > 8.0F)
+	    val[i] = 8.0F;
       }
    }
 }
@@ -268,7 +261,7 @@ handle_pass_op(struct atifs_machine *machine, struct atifs_setupinst *texinst,
    if (pass_tex >= GL_TEXTURE0_ARB && pass_tex <= GL_TEXTURE7_ARB) {
       pass_tex -= GL_TEXTURE0_ARB;
       COPY_4V(machine->Registers[idx],
-	      span->array->texcoords[pass_tex][column]);
+	      span->array->attribs[FRAG_ATTRIB_TEX0 + pass_tex][column]);
    }
    else if (pass_tex >= GL_REG_0_ATI && pass_tex <= GL_REG_5_ATI) {
       pass_tex -= GL_REG_0_ATI;
@@ -286,11 +279,12 @@ handle_sample_op(GLcontext * ctx, struct atifs_machine *machine,
 /* sample from unit idx using texinst->src as coords */
    GLuint swizzle = texinst->swizzle;
    GLuint coord_source = texinst->src;
-   GLfloat tex_coords[4];
+   GLfloat tex_coords[4] = { 0 };
 
    if (coord_source >= GL_TEXTURE0_ARB && coord_source <= GL_TEXTURE7_ARB) {
       coord_source -= GL_TEXTURE0_ARB;
-      COPY_4V(tex_coords, span->array->texcoords[coord_source][column]);
+      COPY_4V(tex_coords,
+              span->array->attribs[FRAG_ATTRIB_TEX0 + coord_source][column]);
    }
    else if (coord_source >= GL_REG_0_ATI && coord_source <= GL_REG_5_ATI) {
       coord_source -= GL_REG_0_ATI;
@@ -325,7 +319,8 @@ execute_shader(GLcontext *ctx, const struct ati_fragment_shader *shader,
    struct atifs_instruction *inst;
    struct atifs_setupinst *texinst;
    GLint optype;
-   GLint i, j, pass;
+   GLuint i;
+   GLint j, pass;
    GLint dstreg;
    GLfloat src[2][3][4];
    GLfloat zeros[4] = { 0.0, 0.0, 0.0, 0.0 };
@@ -348,7 +343,7 @@ execute_shader(GLcontext *ctx, const struct ati_fragment_shader *shader,
 
 	 /* setup the source registers for color and alpha ops */
 	 for (optype = 0; optype < 2; optype++) {
-	    for (i = 0; i < inst->ArgCount[optype]; i++) {
+ 	    for (i = 0; i < inst->ArgCount[optype]; i++) {
 	       GLint index = inst->SrcReg[optype][i].Index;
 
 	       if (index >= GL_REG_0_ATI && index <= GL_REG_5_ATI)
@@ -572,8 +567,8 @@ init_machine(GLcontext * ctx, struct atifs_machine *machine,
 	 machine->Registers[i][j] = 0.0;
    }
 
-   COPY_4V(inputs[ATI_FS_INPUT_PRIMARY], span->array->color.sz4.rgba[col]);
-   COPY_4V(inputs[ATI_FS_INPUT_SECONDARY], span->array->color.sz4.spec[col]);
+   COPY_4V(inputs[ATI_FS_INPUT_PRIMARY], span->array->attribs[FRAG_ATTRIB_COL0][col]);
+   COPY_4V(inputs[ATI_FS_INPUT_SECONDARY], span->array->attribs[FRAG_ATTRIB_COL1][col]);
 }
 
 
@@ -591,8 +586,6 @@ _swrast_exec_fragment_shader(GLcontext * ctx, SWspan *span)
    /* incoming colors should be floats */
    ASSERT(span->array->ChanType == GL_FLOAT);
 
-   ctx->_CurrentProgram = GL_FRAGMENT_SHADER_ATI;
-
    for (i = 0; i < span->end; i++) {
       if (span->array->mask[i]) {
 	 init_machine(ctx, &machine, shader, span, i);
@@ -604,10 +597,8 @@ _swrast_exec_fragment_shader(GLcontext * ctx, SWspan *span)
 	    const GLfloat *colOut = machine.Registers[0];
             /*fprintf(stderr,"outputs %f %f %f %f\n",
               colOut[0], colOut[1], colOut[2], colOut[3]); */
-            COPY_4V(span->array->color.sz4.rgba[i], colOut);
+            COPY_4V(span->array->attribs[FRAG_ATTRIB_COL0][i], colOut);
 	 }
       }
    }
-
-   ctx->_CurrentProgram = 0;
 }
